(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.worker = {}));
}(this, (function (exports) { 'use strict';

  /** @type {any} */var globalThis = typeof global==="object"?global:(typeof window==="object"?window:self)||{};

  let Blob$1=globalThis.Blob;// First attempt to load using Node.js
  if(typeof Blob$1==="undefined"&&undefined){Blob$1=undefined.Blob;}// If still not available, throw an error.
  if(typeof Blob$1==="undefined"){throw new Error("Missing required Blob dependency")}// Extract UUID from object URL generated for an arbitrary blob.
  const getUUID=()=>URL.createObjectURL(new Blob$1).slice(31);

  /**
   * Creates a virtual URL that can be used to create a Worker. This is useful
   * for inlining a script to run, vs using a phsyical file. Since the operations
   * are repetitive we offer this feature as a helper function.
   *
   * @param {Function} fn - Wrapper function containing code to execute in a
   *                        worker
   * @returns {String} URL
   */function toObjectURL(fn){const fnString=String(fn);const firstBracket=fnString.indexOf("{")+1;const lastBracket=fnString.lastIndexOf("}");const workerCode=String(fn).slice(firstBracket,lastBracket);return URL.createObjectURL(// Create a new blob and split the workerCode so it becomes an array with a
  // single entry.
  new Blob(workerCode.split()))}

  /**
   * Finds a more specific type identifier.
   * 
   * @param {any} x 
   * @returns {string}
   */const type=x=>toString.call(x).split(" ")[1].slice(0,-1).toLowerCase();

  /**
   * @returns {DIFF_BINDING}
   */var diff = /** @type {DIFF_BINDING} */ /** @type {any} */globalThis[Symbol.for("diffHTML")];

  const{innerHTML,html}=diff;const{parse}=JSON;const createWebSocket=(socketUrl,{mount,socketOptions})=>{const socket=new WebSocket(socketUrl,socketOptions);socket.addEventListener("message",async e=>{const{type,...rest}=parse(e.data);// TODO Deprecate the 'clear' event. This is currently used in the Node
  // worker when an error is encountered. This cleans up the markup to avoid
  // issues during rendering.
  if(type==="clear"){mount.innerHTML="";}if(type==="patches"){innerHTML(mount,null,{patches:rest.patches});}if(type==="log"){const{level,message}=rest;console[level](...[].concat(message));}});return socket};

  const isNode=typeof global!=="undefined";const SafeWorker=typeof Worker!=="undefined"?Worker:undefined;/**
   *
   * @param {string} workerInit - Location of worker script, Object URL pointing
   * to Blob, or a Worker instance itself.
   * @param {Object} workerOpts
   * @returns {Worker}
   */const createWorker=(workerInit,workerOpts={})=>callback=>{const safeWorkerOpts=workerOpts||{};let worker=null;// If the init is a worker object already, simply assign the value and
  // options.
  if(type(workerInit)==="worker"){worker=workerInit;Object.assign(worker,safeWorkerOpts);}// Otherwise, use the workerInit as the value for creating a new worker.
  else {worker=new SafeWorker(workerInit,{...safeWorkerOpts});}const onMessage=message=>{const{type,...rest}=isNode?message:message.data;if(type==="patches"){callback({type,...rest});}};const onError=error=>{// Extra logging to the Node console, in the browser the error will bubble
  // automatically so this isn't needed.
  if(isNode){console.error(error);}callback({type:"log",level:"error",message:String(error.stack||error.message)});return true};if(isNode){worker.on("message",onMessage);worker.on("error",onError);}else {worker.onmessage=onMessage;worker.onerror=onError;}return worker};

  const{Internals}=diff;const{assign}=Object;const{stringify}=JSON;const linker=new Map;const mainTask=({// TODO Merge socket and worker args together into one named bridge
  // or something similar.
  //
  // WebSocket connection for the main thread.
  socket=null,worker=null}={})=>assign(function webWorkerTask(transaction){const currentTasks=transaction.tasks;const indexOfSyncTrees=currentTasks.indexOf(Internals.tasks.syncTrees);// Only run this middleware when patches are present.
  if(!("patches"in transaction.config)){return}const link=vTree=>{if(vTree&&vTree.childNodes){Internals.Pool.memory.protected.add(vTree);linker.set(vTree.__link,vTree);vTree.childNodes.forEach(x=>link(x));}};// Replace syncTrees with injectPatches
  currentTasks.splice(indexOfSyncTrees,1,function injectPatches(){transaction.patches=transaction.config.patches.map(x=>{// Handle dom events, custom element properties, etc. Any time you need
  // a function, we proxy the call and the arguments.
  if(x&&x.__caller){return function(e){// TODO handled by synthetic events middleware
  e.preventDefault();e.stopPropagation();if(socket){socket.send(stringify({type:"event",...x}));}else {worker.postMessage({type:"event",...x});}}}if(!x||typeof x!=="object"||!("__link"in x)){return x}let vTree=x;if(linker.has(x.__link)){vTree=linker.get(x.__link);return vTree}else if(x.__link==="mount"){vTree=transaction.oldTree;}else {link(vTree);}if((x&&x.isSvg||vTree&&vTree.isSvg)&&vTree){transaction.state.svgElements.add(vTree);}return vTree});});},{releaseHook:vTree=>{if(vTree&&vTree.__link){linker.delete(vTree.__link);}}});

  const{Internals: Internals$1}=diff;const{assign: assign$1}=Object;const linker$1=new WeakMap;const reloadable=new Map;let workerData;const workerTask=({send,callers=new Map,live}={})=>assign$1(function workerTask(transaction){{send=postMessage;}if(transaction.config.skipWorker){return undefined}const currentTasks=transaction.tasks;const indexOfPatchNode=currentTasks.indexOf(Internals$1.tasks.patchNode);const link=patches=>patches.map(x=>{// Already found, return.
  if(linker$1.has(x)){return linker$1.get(x)}if(x&&x.__caller){return {__caller:x.__caller}}if(typeof x==="function"){const __caller=x.__caller||getUUID();callers.set(__caller,x);x.__caller=__caller;return {__caller}}// If x is a protected VTree (meaning it's used)
  if(Internals$1.Pool.memory.protected.has(x)||Internals$1.Pool.memory.allocated.has(x)){// Needs a link added.
  const __link=x===transaction.oldTree?"mount":getUUID();const isSvg=transaction.state.svgElements.has(x)||x.nodeName==="svg";const retVal={__link,isSvg};for(const attrName in x.attributes){const val=x.attributes[attrName];if(typeof val==="function"){const __caller=val.__caller||getUUID();callers.set(__caller,val);val.__caller=__caller;x.attributes[attrName]={__caller};}}linker$1.set(x,retVal);x.isSvg=isSvg;x.__link=__link;return x}return x});// Replace patchNode with skipPatch and return array of patches
  // synchronously
  currentTasks.splice(indexOfPatchNode,1,function skipPatch(){const patches=link(transaction.patches);const{uuid}=transaction.config;send({type:"patches",uuid,patches});transaction.end();return patches});},{subscribe:()=>{//console.log = (message, ...rest) => {
  //  send({ type: 'log', level: 'log', message: [message].concat(rest) });
  //  return oldLog.apply(null, [message].concat(rest));
  //};
  },createTreeHook:vTree=>{if(live&&typeof vTree.rawNodeName==="function"){reloadable.set(vTree,vTree.rawNodeName.displayName||vTree.rawNodeName.name);//console.log('Should reload', vTree.nodeName);
  // Whenever we find a component, run new Error().stack to
  // find the file it came from. Use this to create a mapping
  // to share whenever live is looking up which component
  // changed.
  //
  // From there we can get the component to forceUpdate using
  // the latest component.
  }for(const attrName in vTree.attributes){const val=vTree.attributes[attrName];if(val.__caller){vTree.attributes[attrName]={__caller:val.__caller};}else if(typeof val==="function"){const __caller=getUUID();const fn=vTree.attributes[attrName];vTree.attributes[attrName]={__caller};val.__caller=__caller;callers.set(__caller,fn);}}},workerData,releaseHook:vTree=>linker$1.delete(vTree)});

  exports.createWebSocket = createWebSocket;
  exports.createWorker = createWorker;
  exports.getUUID = getUUID;
  exports.mainTask = mainTask;
  exports.toObjectURL = toObjectURL;
  exports.workerTask = workerTask;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
